package me.mrletsplay.streamdeck;

import java.io.File;
import java.io.IOException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.FloatControl;
import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;

import me.mrletsplay.streamdeck.exception.InitializationException;
import me.mrletsplay.streamdeck.exception.SoundboardException;

public class Soundboard {

	private static ExecutorService executor;

	public static void initialize() throws InitializationException {
		executor = Executors.newCachedThreadPool();
	}

	public static void finish() {
		executor.shutdown();
	}

	public static void play(String path) throws SoundboardException {
		AudioInputStream s;
		try {
			s = AudioSystem.getAudioInputStream(new File("TEST/test.wav"));
		} catch (UnsupportedAudioFileException | IOException e) {
			throw new SoundboardException(e);
		}

		executor.submit(() -> {
			try(Clip clip = AudioSystem.getClip()) {
				Object wait = new Object();
				AtomicBoolean playing = new AtomicBoolean(true);
				clip.addLineListener(event -> {
					if(event.getType() == LineEvent.Type.STOP || event.getType() == LineEvent.Type.CLOSE) {
						playing.set(false);
						wait.notify();
					}
				});

				clip.open(s);
				clip.setFramePosition(0);

//				FloatControl gain = (FloatControl) clip.getControl(FloatControl.Type.MASTER_GAIN);
//				gain.setValue(20f * (float) Math.log10(0.2));

				clip.start();

				while(playing.get()) { wait.wait(); };
			} catch (LineUnavailableException | IOException | InterruptedException e) {
				e.printStackTrace();
			}
		});
	}

	public static void test() throws IOException, UnsupportedAudioFileException, LineUnavailableException {
		AudioInputStream s = AudioSystem.getAudioInputStream(new File("TEST/test.wav"));
		try(Clip c = AudioSystem.getClip()) {
			AtomicBoolean e = new AtomicBoolean(true);
			c.addLineListener(event -> {
				if(event.getType() == LineEvent.Type.STOP) e.set(false);
			});
			c.open(s);

			c.setFramePosition(0);
			FloatControl gain = (FloatControl) c.getControl(FloatControl.Type.MASTER_GAIN);
			gain.setValue(20f * (float) Math.log10(0.2));
			c.start();

			c.loop(10);

			while(e.get());
		}
	}

}
