package me.mrletsplay.streamdeck.action;

import java.util.Collections;
import java.util.List;

import me.mrletsplay.mrcore.json.JSONObject;

public interface DeckAction {

	/**
	 * Useful if parameters can't be directly mapped to JSON types.<br>
	 * By default, this does nothing but may be implemented if needed
	 * @param data The action data
	 * @throws ActionSerializationException If the action can't be deserialized
	 */
	public default void deserialize(JSONObject data) throws ActionSerializationException {}

	/**
	 * Must be implemented to correspond to {@link #deserialize(JSONObject)}.<br>
	 * By default, this does nothing but may be implemented if needed
	 * @param data The action data
	 * @throws ActionSerializationException If the action can't be serialized
	 */
	public default void serialize(JSONObject data) throws ActionSerializationException {}

	/**
	 * Validates the action and returns a list of errors, or an empty list if there are no errors.<br>
	 * By default, this does nothing but may be implemented to add further validation
	 * @return A list of errors, or an empty list if there are no errors
	 */
	public default List<String> validate() {
		return Collections.emptyList();
	}

	/**
	 * Runs the action
	 */
	public void run();

}
